{% extends "base.html" %}

{% block content %}
    <h1>Remove</h1>
    <form method="post">
        <p>Do you really want to remove <strong>{{ path }}</strong> ?</p>
        <input type="submit" value="Yes" /> <a href="{{ backurl }}" alt="Go back">Cancel</a>
    </form>
{% endblock %}
