<!DOCTYPE html>
<html lang="en">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
    <title>{% block title %}{{ config.get("title", "BrowsePy") }}{% endblock %}</title>
    {% block style %}
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='base.css') }}"/>
    {% endblock %}
</head>
<body>
    {% block header %}{% endblock %}
    {% block content %}{% endblock %}
    {% block footer %}{% endblock %}
</body>
</html>
